# noqa: INP001

"""Common settings for LMS and CMS to provide to edX"""


def apply_common_settings(settings):
    """
    Apply custom settings function for LMS and CMS settings.

    Args:
        settings: Django settings object to modify
    """
    settings.ENABLE_AUTO_LANGUAGE_SELECTION = False
    settings.AUTO_LANGUAGE_SELECTION_EXEMPT_PATHS = ["admin", "sysadmin", "instructor"]



# noqa: INP001

"""Settings to provide to edX"""

from ol_openedx_auto_select_language.settings.common import apply_common_settings


def plugin_settings(settings):
    """
    Populate lms settings
    """
    apply_common_settings(settings)
    settings.MIDDLEWARE.extend(
        ["ol_openedx_auto_select_language.middleware.CourseLanguageCookieMiddleware"]
    )
    VIDEO_TRANSCRIPT_LANGUAGE_FILTERS = {
        "org.openedx.learning.xblock.render.started.v1": {
            "pipeline": [
                "ol_openedx_auto_select_language.filters.AddDestLangForVideoBlock"
            ],
            "fail_silently": False,
        }
    }
    existing_filters = getattr(settings, "OPEN_EDX_FILTERS_CONFIG", {})

    # Merge pipeline lists instead of overwriting
    for filter_name, config in VIDEO_TRANSCRIPT_LANGUAGE_FILTERS.items():
        if filter_name not in existing_filters:
            existing_filters[filter_name] = config
        else:
            existing_filters[filter_name]["pipeline"].extend(config.get("pipeline", []))
            # do not override fail_silently
            if "fail_silently" in config:
                existing_filters[filter_name].setdefault(
                    "fail_silently", config["fail_silently"]
                )

    settings.OPEN_EDX_FILTERS_CONFIG = existing_filters



"""
URL configuration for ol_openedx_auto_select_language app.
"""

from django.conf import settings
from django.urls import re_path

from ol_openedx_auto_select_language.views import CourseLanguageView

urlpatterns = [
    re_path(
        rf"auto-select-language/api/course-language/{settings.COURSE_KEY_PATTERN}$",
        CourseLanguageView.as_view(),
        name="ol_course_language",
    ),
    # TODO: Remove the legacy endpoint in a  # noqa: FIX002, TD003, TD002
    #  future release after updating all clients to use the new endpoint.
    re_path(
        rf"course-translations/api/course-language/{settings.COURSE_KEY_PATTERN}$",
        CourseLanguageView.as_view(),
        name="ol_course_language_legacy",
    ),
]



"""
Filters for Open edX auto select language.
"""

from openedx_filters import PipelineStep
from xmodule.modulestore.django import modulestore

from ol_openedx_auto_select_language.constants import (
    ENGLISH_LANGUAGE_CODE,
)
from ol_openedx_auto_select_language.utils import LanguageCode

VIDEO_BLOCK_TYPE = "video"


class AddDestLangForVideoBlock(PipelineStep):
    """
    Pipeline step to add destination language for video transcripts
    """

    def run_filter(self, context, student_view_context):
        """
        Add the destination language to the student view context if a video block
        with transcripts in the course language is found among the child blocks.
        """

        def set_dest_lang_for_video_block(block_key):
            """
            Set the destination language for video blocks with
            transcripts in the course language.
            """
            student_view_context["dest_lang"] = (
                ENGLISH_LANGUAGE_CODE  # default to English
            )
            video_block = modulestore().get_item(block_key)
            transcripts_info = video_block.get_transcripts_info()
            dest_lang = getattr(
                context.get("course", None), "language", ENGLISH_LANGUAGE_CODE
            )
            dest_lang = LanguageCode(dest_lang).to_bcp47()
            if (
                transcripts_info
                and transcripts_info.get("transcripts", {})
                and dest_lang in transcripts_info["transcripts"]
            ):
                student_view_context["dest_lang"] = dest_lang

        block = dict(context)["block"]
        block_usage_key = block.usage_key
        block_type = block_usage_key.block_type
        if block_type == "vertical":
            for child in getattr(block, "children", []):
                if child.block_type == VIDEO_BLOCK_TYPE:
                    set_dest_lang_for_video_block(child)
        elif block_type == VIDEO_BLOCK_TYPE:
            set_dest_lang_for_video_block(block_usage_key)

        return {"context": context, "student_view_context": student_view_context}



"""
API Views for ol_openedx_auto_select_language App
"""

import logging

from opaque_keys import InvalidKeyError
from opaque_keys.edx.keys import CourseKey
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from ol_openedx_auto_select_language.utils import LanguageCode

log = logging.getLogger(__name__)


class CourseLanguageView(APIView):
    """
    API View to retrieve the language of a specified course.

    Sample Request:
        GET /auto-select-language/api/course_language/{course_key}/

    Sample Response:
        200 OK
        {
            "language": "en"
        }

    Error Responses:
        400 Bad Request
        {
            "error": "Invalid course_key."
        }
        404 Not Found
        {
            "error": "Course not found."
        }
        400 Bad Request
        {
            "error": "An unexpected error occurred."
        }
    """

    permission_classes = [IsAuthenticated]

    def get(self, request, course_key_string):  # noqa: ARG002
        """
        Retrieve the language of the specified course.
        """
        try:
            course_key = CourseKey.from_string(course_key_string)
            course = CourseOverview.get_from_id(course_key)
        except InvalidKeyError:
            log.info("Invalid course key %s", course_key_string)
            return Response(
                {"error": "Invalid course_key."},
                status=status.HTTP_400_BAD_REQUEST,
            )
        except CourseOverview.DoesNotExist:
            log.info("Course not found for key %s", course_key_string)
            return Response(
                {"error": "Course not found."},
                status=status.HTTP_404_NOT_FOUND,
            )
        except Exception:
            log.exception("Unexpected error retrieving course %s", course_key_string)
            return Response(
                {"error": "An unexpected error occurred."},
                status=status.HTTP_400_BAD_REQUEST,
            )
        language = LanguageCode(course.language).to_bcp47()
        return Response({"language": language})



# noqa: INP001

"""Settings to provide to edX"""

from ol_openedx_auto_select_language.settings.common import apply_common_settings


def plugin_settings(settings):
    """
    Populate cms settings
    """
    apply_common_settings(settings)
    settings.MIDDLEWARE.extend(
        [
            "ol_openedx_auto_select_language.middleware.CourseLanguageCookieResetMiddleware"
        ]
    )



"""
Utils for ol_openedx_auto_select_language.
"""

import re


class LanguageCode:
    """
    Utility class for handling language code conversions between
    Django/Open edX style and BCP47.
    """

    def __init__(self, lang_code):
        self.lang_code = lang_code

    def to_bcp47(self) -> str:
        """
        Convert Django / Open edX style language codes to BCP47.

        Examples:
            zh_HANS     -> zh-Hans
            zh_HANT     -> zh-Hant
            zh_HANS_CN  -> zh-Hans-CN
            en_US       -> en-US
            es_419      -> es-419
            pt_br       -> pt-BR
        """
        if not self.lang_code:
            return self.lang_code

        parts = self.lang_code.replace("_", "-").split("-")
        result = []
        for idx, part in enumerate(parts):
            if idx == 0:
                # Language
                result.append(part.lower())

            elif re.fullmatch(r"[A-Za-z]{4}", part):
                # Script (Hans, Hant, Latn, Cyrl, etc.)
                result.append(part.title())

            elif re.fullmatch(r"[A-Za-z]{2}", part):
                # Region i.e US, PK, CN
                result.append(part.upper())

            elif re.fullmatch(r"\d{3}", part):
                # Numeric region (419)
                result.append(part)

            else:
                # Variants/extensions
                result.append(part.lower())

        return "-".join(result)

    def to_django(self) -> str:
        """
        Convert BCP47 language tags to Django / Open edX style.

        Examples:
            zh-Hans     -> zh_HANS
            zh-Hant     -> zh_HANT
            zh-Hans-CN  -> zh_HANS_CN
            en-US       -> en_US
            es-419      -> es_419
            pt-BR       -> pt_BR
        """
        if not self.lang_code:
            return self.lang_code

        parts = self.lang_code.replace("_", "-").split("-")
        result = []

        for idx, part in enumerate(parts):
            if idx == 0:
                # Language
                result.append(part.lower())

            elif re.fullmatch(r"[A-Za-z]{4}", part):
                # Script
                result.append(part.upper())

            elif re.fullmatch(r"[A-Za-z]{2}", part):
                # Region
                result.append(part.upper())

            elif re.fullmatch(r"\d{3}", part):
                # Numeric region
                result.append(part)

            else:
                # Variants/extensions
                result.append(part.lower())

        return "_".join(result)



"""
ol_openedx_auto_select_language Django application initialization.
"""

from django.apps import AppConfig
from edx_django_utils.plugins import PluginSettings, PluginURLs
from openedx.core.djangoapps.plugins.constants import ProjectType, SettingsType


class OLOpenEdxAutoSelectLanguageConfig(AppConfig):
    """
    Configuration for the ol_openedx_auto_select_language Django application.
    """

    name = "ol_openedx_auto_select_language"
    verbose_name = "OL Auto Select Language"

    plugin_app = {
        PluginURLs.CONFIG: {
            ProjectType.LMS: {
                PluginURLs.NAMESPACE: "",
                PluginURLs.REGEX: "",
                PluginURLs.RELATIVE_PATH: "urls",
            }
        },
        PluginSettings.CONFIG: {
            ProjectType.CMS: {
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "settings.cms"},
            },
            ProjectType.LMS: {
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "settings.lms"},
            },
        },
    }



"""
Middleware to set/reset language preference cookie and
user preference based on course language.
"""

import re

from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.deprecation import MiddlewareMixin
from opaque_keys.edx.keys import CourseKey
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview
from openedx.core.djangoapps.lang_pref import LANGUAGE_KEY
from openedx.core.djangoapps.lang_pref import helpers as lang_pref_helpers
from openedx.core.djangoapps.user_api.preferences.api import set_user_preference

from ol_openedx_auto_select_language.constants import ENGLISH_LANGUAGE_CODE
from ol_openedx_auto_select_language.utils import LanguageCode


def should_process_request(request):
    """
    Return True if language auto-selection should run for this request.
    """
    return (
        settings.ENABLE_AUTO_LANGUAGE_SELECTION
        and hasattr(request, "user")
        and request.user.is_authenticated
    )


def set_language(request, response, language):
    """
    Set both cookie and user preference for language.
    """
    lang_pref_helpers.set_language_cookie(request, response, language)
    set_user_preference(request.user, LANGUAGE_KEY, language)


def redirect_current_path(request):
    """
    Redirect to the same URL to ensure language change takes effect.
    """
    return HttpResponseRedirect(request.get_full_path())


class CourseLanguageCookieMiddleware(MiddlewareMixin):
    """
    LMS middleware that:
    - Sets language based on course language
    - Forces English for exempt paths and authoring MFEs
    """

    COURSE_URL_REGEX = re.compile(
        rf"^/courses/(?P<course_key>{settings.COURSE_KEY_REGEX})(?:/|$)",
        re.IGNORECASE,
    )

    def process_response(self, request, response):
        """
        Process the response to set/reset language cookie based on course language.
        """
        if not should_process_request(request):
            return response

        path = getattr(request, "path_info", request.path)

        if self._should_force_english(request, path):
            return self._force_english_if_needed(request, response)

        course_language = self._get_course_language(path)
        if not course_language:
            return response

        return self._apply_course_language(request, response, course_language)

    def _should_force_english(self, request, path):
        """
        Determine if English should be forced based on request origin or exempt paths.
        """
        return request.META.get(
            "HTTP_ORIGIN"
        ) == settings.COURSE_AUTHORING_MICROFRONTEND_URL or any(
            exempt_path in path
            for exempt_path in settings.AUTO_LANGUAGE_SELECTION_EXEMPT_PATHS
        )

    def _force_english_if_needed(self, request, response):
        """
        Force language to English if not already set.
        """
        cookie_val = lang_pref_helpers.get_language_cookie(request)

        if cookie_val != ENGLISH_LANGUAGE_CODE:
            set_language(request, response, ENGLISH_LANGUAGE_CODE)
            return redirect_current_path(request)

        return response

    def _get_course_language(self, path):
        """
        Extract course language from the course URL path.
        """
        match = self.COURSE_URL_REGEX.match(path)
        if not match:
            return None

        try:
            course_key = CourseKey.from_string(match.group("course_key"))
            overview = CourseOverview.get_from_id(course_key)
        except Exception:  # noqa: BLE001
            return None

        return getattr(overview, "language", None)

    def _apply_course_language(self, request, response, language):
        """
        Apply the course language if it differs from the current cookie value.
        """
        language = LanguageCode(language).to_bcp47()
        cookie_val = lang_pref_helpers.get_language_cookie(request)
        if cookie_val != language:
            set_language(request, response, language)
            return redirect_current_path(request)

        return response


class CourseLanguageCookieResetMiddleware(MiddlewareMixin):
    """
    CMS middleware that always resets language to English.
    """

    def process_response(self, request, response):
        """
        Process the response to reset language cookie to English.
        """
        if not should_process_request(request):
            return response

        cookie_val = lang_pref_helpers.get_language_cookie(request)
        if cookie_val and cookie_val != ENGLISH_LANGUAGE_CODE:
            set_language(request, response, ENGLISH_LANGUAGE_CODE)

        return response
